#include "DistrhoPlugin.hpp"
#include <array>

constexpr unsigned int NUM_VOICES = 512;
constexpr unsigned int VOICE_MASK = NUM_VOICES - 1;

class Synth
{
public:
    explicit Synth(double sampleRate);
    void process(float *output, uint32_t frames);
    void setPhaseVelocity(double in);
    void setPitchOffset(double in);
    void setSampleRate(double oldRate, double newRate);
    void setVolume(double in);

    //For recordable performances.
    //For now it requires a button push, but perhaps
    //we should require time position and have the phase reset automatically
    //on song start?
    void resetPhase();


    //Current fundamental frequency of blit.
    double hzFund = fMin;

private:
    void shiftUp();
    void shiftDown();
    //Phase of wavetable.
    double tablePhase = 0.0;
    float volume = 0.0f;
    //Parameter in unit circle controlling pitch (varies by one octave).
    double spectrumPhase = 1.0f;
    //Revolutions per second of spectrum phase.
    double spectrumPhaseVelocity = 0.0;
    //Lowest fundamental frequency of blit.
    double fMin = 55.0;

    //Parameter stretching the overtone series.
    double inharmonicity = 1.001;

    double hzNyq = 24000.0;
    double sampleInterval = 1.0 / 48000.0;
};