#include "DistrhoUI.hpp"
#include "tablet.h"
#include "scale.h"
#ifdef DEBUG
#include <format>
#include <optional>
#endif

START_NAMESPACE_DISTRHO

class ButtonMappingWidget : public NanoSubWidget,
                            public ButtonEventHandler
{
public:
    explicit ButtonMappingWidget(
        Widget *parent,
        float initialSize,
        float initialX,
        float initialY,
        Parameters associatedParameter,
        ButtonEventHandler::Callback *const callback);
    void onNanoDisplay() override;
    bool onMouse(const MouseEvent &ev) override;
    void setMask(const ButtonMask m) { mask = m; };
    bool matchesMask(const ButtonMask m) { return isPenPressed = (m && mask == m); };

    bool isClicked;
    bool isPenPressed;

private:
    float x = 0;
    float y = 0;
    float size = 0;
    const Parameters param;
    ButtonMask mask;
};

class TabUI : public UI,
              public ButtonEventHandler::Callback
{
public:
    explicit TabUI();

    void buttonClicked(SubWidget *const widget, int) override;
    void parameterChanged(uint32_t index, float value) override;

    void uiIdle() override;
    bool onMouse(const MouseEvent &ev) override;
    bool onScroll(const ScrollEvent &ev) override;
    void onResize(const ResizeEvent &ev) override;
    void onNanoDisplay() override;

private:
    void getTabletData();
    void setButtonsValue(unsigned long buttonMask);
    void drawCircle(float x, float y, float z, float p);

    // Tablet context handler
    Tablet tab;
    Packet pkt = {0};
    Packet lastPkt = {0};

    // Button mapping widgets
    ButtonMappingWidget AButtonWidget;
    ButtonMappingWidget BButtonWidget;

    // For displaying frequency.
    float hz = 55.f;
    Scale scale{440.0};

    DISTRHO_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR(TabUI)
};

END_NAMESPACE_DISTRHO