#include "DistrhoPlugin.hpp"
#include "filter.h"

START_NAMESPACE_DISTRHO

class ShepPlug : public Plugin
{
public:
	ShepPlug()
		: Plugin(kParameterCount, 0, 0),
		hzNyq(0.5 * getSampleRate())
	{
		// clear all parameters
		std::memset(fParameters, 0, sizeof(float) * kParameterCount);

	}

protected:
	const char* getLabel() const override { return "yaw-vowel"; }
	const char* getDescription() const override { return "Drawing tablet synth UI"; }
	const char* getMaker() const override { return "yaw-audio"; }
	const char* getHomePage() const override { return "https://yaw.man/plugins/yaw-vowel"; }
	const char* getLicense() const override { return "Fuck you pay me"; }
	uint32_t    getVersion() const override { return d_version(1, 0, 0); }
	int64_t     getUniqueId() const override { return d_cconst('y', 'v', 'w', 'l'); }


	void initParameter(uint32_t index, Parameter& parameter) override
	{
		parameter.hints = kParameterIsAutomable;
		parameter.ranges.def = 0.0f;
		parameter.ranges.min = 0.0f;
		parameter.ranges.max = 1.0f;

		switch (index)
		{
		case ktpax:
			parameter.name = "x";
			parameter.symbol = "x";
			break;
		case ktpay:
			parameter.name = "y";
			parameter.symbol = "y";
			break;
		case ktpaz:
			parameter.name = "z";
			parameter.symbol = "z";
			break;
		case ktpap:
			parameter.name = "p";
			parameter.symbol = "p";
			break;
		case kParameterButtonA:
			parameter.name = "Button A";
			parameter.symbol = "A";
			parameter.hints |= kParameterIsBoolean;
			break;
		case kParameterButtonB:
			parameter.name = "Button B";
			parameter.symbol = "B";
			parameter.hints |= kParameterIsBoolean;
			break;
		case kParameterTime:
			parameter.name = "t";
			parameter.symbol = "t";
			parameter.hints = kParameterIsOutput;
			break;
		}
	}

	void sampleRateChanged(double newRate) override
	{
		hzNyq = newRate / 2;
	}

	float getParameterValue(uint32_t index) const override
	{
		return fParameters[index];
	}

	void setParameterValue(uint32_t idx, float val) override
	{
		fParameters[idx] = val;
		if( kParameterTime == idx ) return;
		double x = (fParameters[ktpax] * 700.0 + 200.0) / (2.0 * hzNyq);
		double y = (fParameters[ktpay] * 1900.0 + 500.0) / (2.0 * hzNyq);
		double p = fParameters[ktpap];
		filter.set(x, y, p);
	}

	void run(const float** inputs, float** outputs, uint32_t frames) override
	{
		filter.process(outputs, inputs, frames);
		parity = !parity;
		fParameters[kParameterTime] += parity ? 1 : -1;
	}

private:
	float fParameters[kParameterCount];
	double hzNyq;
	bool parity = false;
	Filter filter;

	DISTRHO_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR(ShepPlug)
};

Plugin* createPlugin()
{
	return new ShepPlug();
}

END_NAMESPACE_DISTRHO
