#include "DistrhoPlugin.hpp"
#include <tuple>
#include <array>

class Resonator
{
public:

    inline float process(float x);
    void clear();
    void set(double, double, double, double);

private:
    float xp = 0.0;
    float xpp = 0.0;
    float yp = 0.0;
    float ypp = 0.0;

    double cypp = 0.0;
    double cyp = 0.0;
    double cxp = 0.0;
    double cx = 1.0;
};

class Filter
{
public:
    Filter();
    void process(float **outputs, const float **inputs, uint32_t frames);
    void set(double x, double y, double p);

private:
    std::array<std::array<Resonator, 2>, 2> res;
};


//Uncapped resonators
//Implemented in parallel
//Use partial fraction expansion to ensure constant numerator.
template< uint N >
class ParallelResonators
{
public:
    void process(float **outputs, const float **inputs, uint32_t frames);
    void set(std::array<double, N> &hz, std::array<double, N> &bw);

    private:
    double scale;
    std::array<std::array<Resonator, N>, 2> res{};
};